from __future__ import print_function
import pytest
import subprocess as sp
from virgotools.cm_lib import *
from virgotools import cm_lib
from virgotools.pvt_utils import ascii, native_str

from virgotools.virgo_logging import log_to_console
log_to_console('DEBUG')

my_cm_name = 'pytest_test_cm_lib'


def cm_names_cli():
    """run 'cm names' on command line and return as list"""
    result = sp.check_output(cm_lib.CM_COMMAND + ' names', shell=True)
    result = native_str(result)
    return result.splitlines()


def setup_module():
    """Cm uses global state, do some tests before"""
    print('\nIn setup_module')

    # check that cm names is working
    names = cm_names_cli()
    assert 'CmNameServer' in names

    # test that my name does not exist yet
    assert my_cm_name not in names
    assert cm_lib._my_cm_name is None

    # sending message before running cm_start should fail
    with pytest.raises(CmError):
        cm_send('CmNameServer', 'NSGetNames')

    cm_start(my_cm_name)
    assert cm_lib._my_cm_name == ascii(my_cm_name)


def test_start_with_same_name():
    cm_start(my_cm_name)


def test_change_name():
    cm_start('SomeOtherName')
    assert cm_lib._my_cm_name == ascii(my_cm_name)


def test_cm_names():
    names = cm_send_receive('NSNames', 1, 'CmNameServer', 'NSGetNames')
    assert names == cm_names_cli()[:-1]  # last item is cm_1 from cli process
    assert 'CmNameServer' in names
    assert my_cm_name in names


def test_unknown_server():
    with pytest.raises(CmError):
        cm_send('NonExistingServer', 'NonExistingType')
